--------------------------------------------------------------------------------
-- Company: Trenz Electronic GmbH
-- Engineer: Oleksandr Kiyenko
--------------------------------------------------------------------------------
library ieee;
use ieee.STD_LOGIC_1164.all;
use ieee.numeric_std.all;
--------------------------------------------------------------------------------
entity bcd7seg is
generic (
	CLK_DIVISOR		: integer := 500000
);
port (
	clk				: in  STD_LOGIC;
	-- Inputs
	D				: in  STD_LOGIC_VECTOR(15 downto 0);
	P				: in  STD_LOGIC_VECTOR( 3 downto 0);
	L				: in  STD_LOGIC_VECTOR( 2 downto 0);
	-- 7-segment display Interface
	A				: out STD_LOGIC_VECTOR( 4 downto 0);
	C				: out STD_LOGIC_VECTOR( 7 downto 0)
);
end bcd7seg;
--------------------------------------------------------------------------------
architecture arch_imp of bcd7seg is
--------------------------------------------------------------------------------
function bcd2leds(data : in STD_LOGIC_VECTOR(3 downto 0)) return STD_LOGIC_VECTOR is
variable ret		: STD_LOGIC_VECTOR(6 downto 0);
begin
	case data(3 downto 0) is
		when "0000" => ret := "1000000"; -- "0"
		when "0001" => ret := "1111001"; -- "1"
		when "0010" => ret := "0100100"; -- "2"
		when "0011" => ret := "0110000"; -- "3"
		when "0100" => ret := "0011001"; -- "4"
		when "0101" => ret := "0010010"; -- "5"
		when "0110" => ret := "0000010"; -- "6"
		when "0111" => ret := "1111000"; -- "7"
		when "1000" => ret := "0000000"; -- "8"
		when "1001" => ret := "0010000"; -- "9"
		when "1010" => ret := "0001000"; -- "A"
		when "1011" => ret := "0000011"; -- "B"
		when "1100" => ret := "0001110"; -- "C"
		when "1101" => ret := "0100001"; -- "D"
		when "1110" => ret := "0000110"; -- "E"
		when "1111" => ret := "0001110"; -- "F"
	end case;
    return ret;
end;
signal cycle_cnt	: INTEGER range 0 to CLK_DIVISOR - 1;
signal segsel		: INTEGER range 0 to 4;
--------------------------------------------------------------------------------
begin
--------------------------------------------------------------------------------
process(clk)
begin
	if(clk = '1' and clk'event)then
		if(cycle_cnt = (CLK_DIVISOR - 1))then
			cycle_cnt	<= 0;
			if(segsel = 4)then
				segsel	<= 0;
			else
				segsel	<= segsel + 1;
			end if;
		else
			cycle_cnt 	<= cycle_cnt + 1;
		end if;
		case segsel is
			when 0 => 
				A		<= "10000";
				C		<= (not P(0)) & bcd2leds(D( 3 downto  0));
			when 1 => 
				A		<= "01000";
				C		<= (not P(1)) & bcd2leds(D( 7 downto  4));
			when 2 => 
				A		<= "00100";
				C		<= (not P(2)) & bcd2leds(D(11 downto  8));
			when 3 => 
				A		<= "00010";
				C		<= (not P(3)) & bcd2leds(D(15 downto 12));
			when 4 => 
				A		<= "00001";
				C		<= "00000" & (not L);
		end case;
	end if;
end process;
--------------------------------------------------------------------------------
end arch_imp;
